//================================
//Exposed Functions.
//Intended for use by your code.
//================================

//If your timer returns this, it will delete the timer.
CONST.TIMER_DELETE <- INT_MAX - 1;

//You can skip `order` parameter. Default value is 0.
::AddTimer <- function(interval, order, ...)
{
    if (typeof(order) == "function")
        return ::AddTimer.acall([this, interval, 0, order].extend(vargv));

    local timerFunc = vargv.remove(0);
    local scope = timerFunc.getinfos().parameters.len() - 1 < vargv.len()
        ? vargv.pop()
        : null;

    if (!scope)
        scope = this;
    else if ("IsValid" in scope)
    {
        scope.ValidateScriptScope();
        scope = scope.GetScriptScope();
    }

    local timerEntry = [order, timerFunc, vargv, scope.weakref(), clampFloor(0, interval), Time()];

    local i = 0;
    for (; i < lizardTimersLen; i++)
        if (::lizardTimers[i][0] < order)
            break;
    ::lizardTimers.insert(i, timerEntry);
    ::lizardTimersLen++;

    return timerEntry;
}
::OnTimer <- AddTimer;

::RunWithDelay <- function(delay, delayedFunc, ...)
{
    local scope = delayedFunc.getinfos().parameters.len() - 1 < vargv.len()
        ? vargv.pop()
        : null;

    if (scope == null)
        scope = this;
    else if ("IsValid" in scope)
    {
        scope.ValidateScriptScope();
        scope = scope.GetScriptScope();
    }

    ::lizardDelays.push([delayedFunc, vargv, scope.weakref(), Time() + delay]);
    ::lizardDelaysLen++;
}
::Schedule <- RunWithDelay;

::OnTickEnd <- function(delayedFunc, ...)
{
    return RunWithDelay.acall([this, -1, delayedFunc].extend(vargv));
}

::OnNextTick <- function(delayedFunc, ...)
{
    return RunWithDelay.acall([this, 0.01, delayedFunc].extend(vargv));
}

::TickLizardTimers <- function()
{
    EntFireByHandle(self, "RunScriptCode", "TickLizardTimers()", 0.01, null, null);
    local time = Time();
    for (local i = 0; i < lizardTimersLen; i++)
    {
        local entry = ::lizardTimers[i];
        if (!entry[3])
        {
            ::lizardTimers.remove(i--);
            ::lizardTimersLen--;
        }
        else if (time - entry[4] >= entry[5])
        {
            entry[5] += entry[4];
            try
            {
                local result = entry[1].acall([entry[3]].extend(entry[2]));
                if (result == TIMER_DELETE)
                    entry[3] = null;
            }
            catch (e) { }
        }
    }
    for (local i = 0; i < lizardDelaysLen; i++)
    {
        local entry = ::lizardDelays[i];
        if (!entry[2])
        {
            ::lizardDelays.remove(i--);
            ::lizardDelaysLen--;
        }
        else if (time >= entry[3])
        {
            try { entry[0].acall([entry[2]].extend(entry[1])); } catch (e) { }
            ::lizardDelays.remove(i--);
            ::lizardDelaysLen--;
        }
    }
}